from __future__ import annotations

from pygismeteo_base import models
from pygismeteo_base.periods import mixins
from pygismeteo_base.types import Params, Step3Days, Step6or24Days

from ._abc import Period, StepN


class Current(mixins.CurrentMixin, Period):
    __slots__ = ()

    def by_coordinates(
        self, latitude: float, longitude: float
    ) -> models.current.Model:
        """По координатам.

        Args:
            latitude: Широта (от -90 до 90).
            longitude: Долгота (от -180 до 180).
        """
        url, params = self._get_params_by_coordinates(latitude, longitude)
        return self._get_result(url, params=params)

    def by_id(
        self,
        # pylint: disable-next=invalid-name,redefined-builtin
        id: int,
    ) -> models.current.Model:
        """По ID географического объекта.

        Args:
            id: ID географического объекта. Получить можно через поиск.
        """
        url, params = self._get_params_by_id(id)
        return self._get_result(url, params=params)

    def _get_result(
        self, url: str, *, params: Params = None
    ) -> models.current.Model:
        response = self._session.get_response(url, params=params)
        return self._model.parse_obj(response)


# pylint: disable-next=too-many-ancestors
class Step3(
    mixins.Step3Mixin,
    StepN[Step3Days, models.step3.Model, models.step3.ModelItem],
):
    __slots__ = ()


# pylint: disable-next=too-many-ancestors
class Step6(
    mixins.Step6Mixin,
    StepN[Step6or24Days, models.step6.Model, models.step6.ModelItem],
):
    __slots__ = ()


# pylint: disable-next=too-many-ancestors
class Step24(
    mixins.Step24Mixin,
    StepN[Step6or24Days, models.step24.Model, models.step24.ModelItem],
):
    __slots__ = ()



from __future__ import annotations

from abc import abstractmethod
from typing import Generic, List, Type

from pygismeteo_base.periods.abc import PeriodABC, StepNABC
from pygismeteo_base.types import Params, TDays, TStepNModel, TStepNModelItem

from ._http import RequestsClient


# pylint: disable-next=abstract-method
class Period(PeriodABC):
    __slots__ = ("_session",)

    def __init__(self, session: RequestsClient) -> None:
        self._session = session


class StepN(Generic[TDays, TStepNModel, TStepNModelItem], StepNABC, Period):
    __slots__ = ()

    def by_coordinates(
        self, latitude: float, longitude: float, days: TDays
    ) -> List[TStepNModelItem]:
        """По координатам.

        Args:
            latitude: Широта (от -90 до 90).
            longitude: Долгота (от -180 до 180).
            days: Количество дней
                (с шагом 3 часа - от 1 до 10, с шагом 6 или 24 - от 3 до 10).
        """
        url, params = self._get_params_by_coordinates(
            latitude, longitude, days=days
        )
        return self._get_result(url, params=params)

    def by_id(
        self,
        # pylint: disable-next=invalid-name,redefined-builtin
        id: int,
        days: TDays,
    ) -> List[TStepNModelItem]:
        """По ID географического объекта.

        Args:
            id: ID географического объекта. Получить можно через поиск.
            days: Количество дней
                (с шагом 3 часа - от 1 до 10, с шагом 6 или 24 - от 3 до 10).
        """
        url, params = self._get_params_by_id(id, days=days)
        return self._get_result(url, params=params)

    def _get_result(
        self, url: str, *, params: Params
    ) -> List[TStepNModelItem]:
        response = self._session.get_response(url, params=params)
        model = self._model.parse_obj(response)
        return model.__root__  # type: ignore[return-value]

    @property
    @abstractmethod
    def _model(self) -> Type[TStepNModel]:
        pass  # pragma: no cover



from __future__ import annotations

from typing import Any, Optional

from pygismeteo_base.http import BaseHttpClient
from pygismeteo_base.types import Params
from pygismeteo_base.validators import Settings
from requests import Session


class RequestsClient(BaseHttpClient):
    __slots__ = ("session",)

    def __init__(self, session: Optional[Session], settings: Settings) -> None:
        super().__init__(settings)
        self.session = session

    def get_response(self, endpoint: str, *, params: Params = None) -> Any:
        response = self._get_json(endpoint, params=params)
        return response["response"]

    def _get_json(self, endpoint: str, *, params: Params = None) -> Any:
        if isinstance(self.session, Session):
            return self._fetch(endpoint, params=params, session=self.session)
        with Session() as session:
            return self._fetch(endpoint, params=params, session=session)

    def _fetch(
        self, endpoint: str, *, params: Params, session: Session
    ) -> Any:
        params, headers = self._get_params_and_headers(params)
        with session.get(
            f"https://api.gismeteo.net/v2/{endpoint}/",
            params=params,
            headers=headers,
        ) as response:
            response.raise_for_status()
            return response.json()



from __future__ import annotations

from typing import Any, List

from pygismeteo_base import models, search
from pygismeteo_base.types import Params, SearchLimit

from ._http import RequestsClient


class Search(search.Search):
    __slots__ = ("_session",)

    def __init__(self, session: RequestsClient) -> None:
        self._session = session

    def by_coordinates(
        self, latitude: float, longitude: float, limit: SearchLimit
    ) -> List[models.search_by_coordinates.ModelItem]:
        """Поиск по координатам.

        Args:
            latitude: Широта (от -90 до 90).
            longitude: Долгота (от -180 до 180).
            limit: Ограничение количества (от 1 до 36).
        """
        params = self._get_params_by_coordinates(
            latitude=latitude, longitude=longitude, limit=limit
        )
        response = self._get_response(params)
        model = models.search_by_coordinates.Model.parse_obj(response)
        return model.__root__

    def by_ip(self, ip: str) -> models.search_by_ip.Model:
        """Поиск по IP-адресу.

        Args:
            ip: IP-адрес.
        """
        params = self._get_params_by_ip(ip)
        response = self._get_response(params)
        return models.search_by_ip.Model.parse_obj(response)

    def by_query(self, query: str) -> List[models.search_by_query.ModelItem]:
        """Поиск по строке.

        Args:
            query: Город, район, область, страна или аэропорт.
        """
        params = self._get_params_by_query(query)
        response = self._get_response(params)
        model = models.search_by_query.Model.parse_obj(response["items"])
        return model.__root__

    def _get_response(self, params: Params) -> Any:
        return self._session.get_response(self._endpoint, params=params)



from __future__ import annotations

from typing import Optional

from pygismeteo_base.types import Lang
from pygismeteo_base.validators import Settings
from requests import Session

from ._http import RequestsClient
from ._periods import Current, Step3, Step6, Step24
from ._search import Search


class Gismeteo:
    """Обёртка для Gismeteo API."""

    __slots__ = (
        "_settings",
        "_session",
        "_current",
        "_step3",
        "_step6",
        "_step24",
        "_search",
    )

    def __init__(
        self,
        *,
        lang: Optional[Lang] = None,
        token: Optional[str] = None,
        session: Optional[Session] = None,
    ) -> None:
        """Обёртка для Gismeteo API.

        Args:
            lang: язык. По умолчанию "ru".
            token: X-Gismeteo-Token,
                если используемый по умолчанию перестал работать.
            session: экземпляр requests.Session.
                По умолчанию для каждого запроса создаётся новый экземпляр.
        """
        self._settings = Settings(lang=lang, token=token)
        self._session = RequestsClient(session, self._settings)
        self._current = Current(self._session)
        self._step3 = Step3(self._session)
        self._step6 = Step6(self._session)
        self._step24 = Step24(self._session)
        self._search = Search(self._session)

    @property
    def session(self) -> Optional[Session]:
        return self._session.session

    @session.setter
    def session(self, session: Optional[Session]) -> None:
        self._session.session = session

    @property
    def lang(self) -> Optional[Lang]:
        """Язык."""
        return self._settings.lang

    @lang.setter
    def lang(self, lang: Optional[Lang]) -> None:
        self._settings.lang = lang

    @property
    def token(self) -> Optional[str]:
        """X-Gismeteo-Token."""
        return self._settings.token

    @token.setter
    def token(self, token: Optional[str]) -> None:
        self._settings.token = token

    @property
    def current(self) -> Current:
        """Текущая погода."""
        return self._current

    @property
    def step3(self) -> Step3:
        """Погода с шагом 3 часа."""
        return self._step3

    @property
    def step6(self) -> Step6:
        """Погода с шагом 6 часов."""
        return self._step6

    @property
    def step24(self) -> Step24:
        """Погода с шагом 24 часа."""
        return self._step24

    @property
    def search(self) -> Search:
        """Поиск."""
        return self._search



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pygismeteo']

package_data = \
{'': ['*']}

install_requires = \
['pygismeteo-base>=3.0,<4.0', 'requests>=2.28,<3.0']

setup_kwargs = {
    'name': 'pygismeteo',
    'version': '5.0.4',
    'description': 'Wrapper for Gismeteo API',
    'long_description': '# pygismeteo\n\n[![CI](https://github.com/monosans/pygismeteo/actions/workflows/ci.yml/badge.svg?branch=main&event=push)](https://github.com/monosans/pygismeteo/actions/workflows/ci.yml)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/monosans/pygismeteo/main.svg)](https://results.pre-commit.ci/latest/github/monosans/pygismeteo/main)\n[![codecov](https://codecov.io/gh/monosans/pygismeteo/branch/main/graph/badge.svg)](https://codecov.io/gh/monosans/pygismeteo)\n\nОбёртка для [Gismeteo API](https://gismeteo.ru/api/).\n\nАсинхронная версия [здесь](https://github.com/monosans/aiopygismeteo).\n\n## Установка\n\n```bash\npython -m pip install -U pygismeteo\n```\n\n## Документация\n\n[pygismeteo.readthedocs.io](https://pygismeteo.readthedocs.io/)\n\n## Пример, выводящий температуру в Москве сейчас\n\n```python\nfrom pygismeteo import Gismeteo\n\ngismeteo = Gismeteo()\nsearch_results = gismeteo.search.by_query("Москва")\ncity_id = search_results[0].id\ncurrent = gismeteo.current.by_id(city_id)\nprint(current.temperature.air.c)\n```\n\n## License / Лицензия\n\n[MIT](https://github.com/monosans/pygismeteo/blob/main/LICENSE)\n',
    'author': 'monosans',
    'author_email': 'hsyqixco@protonmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/monosans/pygismeteo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
